import sys, pprint, logging
import xml.dom.pulldom
from vipus.dag.condor import tailfileobject
from vipus.dag import events

from vipus.logUtil import logger

## Adapter to support Condor's XML log file format (SAX won't parse a
## document which is a sequence of <c>stuff</c> tags, the first tag in
## an XML document is considered the 'document element' and can't be
## repeated.  So we wrap the tailfile object with an extra <xml> tag at the beginning

class tailfileobject_condorxml(tailfileobject.tailfileobject):
    document_element = "<xml>"
    def __init__(self, name, terminate, mode='r', buffering=0):
        tailfileobject.tailfileobject.__init__(self, name, terminate, mode, buffering)
        self.__xml_current_position = 0
        self.done = 0

    ## Don't look at this function too closely, it will cause blindness.
    def read(self, size):
        if self.done:
            logger.info("Terminate condition, done")
            return ''
        
        ## if we've read beyond the document_element at the beginning, just produce data from the real file
        if self.__xml_current_position > len(tailfileobject_condorxml.document_element):
            data = tailfileobject.tailfileobject.read(self, size)
        ## produce some bytes from the document_element plus produce data from the real file
        elif self.__xml_current_position + size > len(tailfileobject_condorxml.document_element):
            data = tailfileobject_condorxml.document_element[self.__xml_current_position:]
            data += tailfileobject.tailfileobject.read(self, size - len(data))
        ## unlikely case of small read at the beginning only within the document element
        else: ## self.__xml_current_position <= len(tailfileobject_condorxml.document_element)
            data = tailfileobject_condorxml.document_element[self.__xml_current_position:self.__xml_current_position + size]

        if data == '':
            logger.info("Terminate condition, returning </xml>")
            self.done = 1
            return "</xml>"
        self.__xml_current_position += size
        return data

class ParseDAGManXMLLog:
    def __init__(self, parser, nveClient, jobDict):
        ## The parser object (so we can force it to finish reading)
        self.parser = parser
        ## The window object we post events to
        self.nveClient = nveClient
        ## The dictionary of job names to job objects
        self.jobDict = jobDict
        
        self._events = xml.dom.pulldom.parse(parser)

    def parse(self):
        self.jobMapping = {}
        
        for event, node in self._events:
            if event == 'START_ELEMENT' and node.nodeName == 'c':
                logger.info("Node: %s" % node)
                self._events.expandNode(node)
                children = node.getElementsByTagName('a')
                stuff = {}
                for child in children:
                    ## str() to remove unicode
                    name = str(child.getAttribute('n'))
                    child_type = child.firstChild.nodeName
                    if child_type == 'b':
                        child_value = child.firstChild.getAttribute('v') == 't'
                    else:
                        ## warning: currently doesn't work for "<a n="SubmitHost"><s>&lt;131.243.2.147:35226&gt;</s></a>":
                        ## produces only '>' :-(
                        if child_type == 'i':
                            child_value = int(child.firstChild.firstChild.data)
                        elif child_type == 's':
                            ## str() to remove unicode
                            child.firstChild.normalize()
                            child_value = str(child.firstChild.firstChild.nodeValue)
                        elif child_type == 'r':
                            child_value = float(child.firstChild.firstChild.data)
                        else:
                            raise RuntimeError, "Unsupported type:", child_type
                    logger.info("%s: %s" % (name, child_value))
                    stuff[name] = child_value
                if stuff['MyType'] == 'SubmitEvent':
                    self.jobMapping[stuff['Cluster']] = self.jobDict[stuff['LogNotes'][10:]]
                    stuff['job'] = self.jobDict[stuff['LogNotes'][10:]]
                else:
                    stuff['job'] = self.jobMapping[stuff['Cluster']]
                    
                logger.info("\n")

                classtype = stuff['MyType']
                if classtype in events.__dict__:
                    classobject = events.__dict__[classtype]
                else:
                    classobject = events.UnsupportedEvent
                args = ()
                new_class = classobject(*args, **stuff)
                logger.info("new class instance %s", pprint.pformat(new_class.__dict__))
                if self.nveClient:
                    self.nveClient.postCondorResults(new_class)

            
if __name__ == '__main__':
    # Open the Condor XML output file
    import Queue
    terminate = Queue.Queue()
    tfo = tailfileobject_condorxml('default.log', terminate)

    # Create the handler
    jobDict={}
    jobDict['A'] = "jobA"
    jobDict['B'] = "jobB"
    dh = ParseDAGManXMLLog(tfo, None, jobDict)
    dh.parse()
